# coding:utf-8

require 'rubygems'

require 'pp'
require 'securerandom'
require 'logger'
require 'yaml'
require 'json'
require 'tempfile'
require 'thread'
require 'ginbee/web_socket'

module PushExchanger

  def self.start(conf)
    core = Core.new conf
    core.start
  end

  class Core

    RETRY_WAIT_INIT = 2
    RETRY_WAIT_MAX  = 600

    def initialize(conf)
      @conf = conf
      @logger = Logger.new(@conf[:log_file],7,1024*1024*10)
      @conf[:logger] = @logger
      
      case @conf[:log_level]
      when "debug"
        @logger.level = Logger::DEBUG
      else
        @logger.level = Logger::INFO
      end 
      
      @plugin_map = load_plugin
      @tempkey = make_cert_key(@conf[:cert_key],@conf[:passphrase]) if @conf[:use_ssl]
      @pid = nil
      @retry_wait_sec = RETRY_WAIT_INIT
    end

    def start
      Signal.trap(:INT) do
        puts "trap int"
        if @pid
          Process.kill(:INT,@pid)
          Process.waitall
        end
        exit!
      end
      
      loop do
        @pid = myfork do
          begin
            exec_receiver
          rescue Exception => ex
            @logger.info(ex)
            puts ex
            puts ex.backtrace
          end
        end
        Process.waitall if @pid
        sleep(@retry_wait_sec)
        @retry_wait_sec *= 2
        @retry_wait_sec = RETRY_WAIT_MAX if @retry_wait_sec > RETRY_WAIT_MAX
        @logger.info("next wait sec:#{@retry_wait_sec}")
      end
    end

    private

    def load_plugin
      map = {}
      plugins = []
      dirs = @conf[:plugin_load_path] || []
      dirs.each do |d|
        Dir.glob(File.join(d,"*/")) do |dir|
          begin
            name = File.basename(dir)
            require "#{dir}/#{name}.rb"
            name[0] = name[0].upcase
            plugins << Module.const_get("#{name}").new(@conf)
            @logger.info("plugin #{name} loaded")
          rescue Exception => ex
            @logger.info("plugin #{name} load error")
            @logger.info(ex.message)
          end
        end
      end
      plugins.each do |plg|
        begin
          plg.get_events.each do |e|
            map[e] = [] unless map[e]
            map[e] << plg
          end
        rescue Exception => ex
          @logger.info("plugin #{plg} get_events error")
          @logger.info(ex)
        end
      end
      map
    end
    
    def make_cert_key(cert_key,pass)
      key = OpenSSL::PKey::RSA.new(File.read(cert_key),pass)
      temp = Tempfile.new(SecureRandom.uuid)
      temp.puts(key.export)
      temp.close
      temp
    end
    
    def exec_receiver
      EM.run do
        
        subid = nil
        auth = ["user:#{@conf[:api_key]}"].pack('m').delete("\r\n")
        option = {
          :headers => {
            "Authorization"      => "Basic #{auth}",
            "X-GINBEE-TENANT-ID" => "1"},
          :ping    => 10
        }
        if @conf[:use_ssl]
          option[:tls] = {
            :ca_file          => @conf[:ca_cert],
            :cert_chain_file  => @conf[:cert],
            :private_key_file => @tempkey.path,
            :verify_peer      => true,
          }
        end
        ws = Ginbee::WebSocket::Client.new(@conf[:ws_server],nil,option)
      
        ws.on :open do |event|
          @logger.info "open start"
          reqid = SecureRandom.uuid
          str = %|{"command":"subscribe","req.id":"#{reqid}","event":"*"}|
          ws.send str
          @logger.info "open end"
          @retry_wait_sec = RETRY_WAIT_INIT
          @logger.info "reset retry wait sec:#{@retry_wait_sec}"
        end
      
        ws.on :message do |event|
          puts event.data
          @logger.info event.data
          m = JSON.parse(event.data)
          command = m["command"]
          case command
          when "subscribed"
            subid = m["sub.id"]
          when "event"
            data = m["data"]
            event = data["event"]
            body  = data["body"]
            @logger.info "event:#{event}"
            if @plugin_map[event]
              @plugin_map[event].each do |plg|
                Thread.new do
                  begin
                    @logger.info("call #{plg} for #{event}")
                    plg.send(event,data)
                  rescue Exception => ex
                    @logger.info(ex)
                  end
                end
              end
            end
          end
        end
    
        ws.on :error do |event|
          puts "error"
          @logger.info "error event:#{event}"
        end
    
        ws.on :close do |event|
          puts "close code:#{event.code} reason:#{event.reason}"
          @logger.info "close code:#{event.code} reason:#{event.reason}"
          ws = nil
          raise "close"
        end
      end
    end
    
    def myfork
      pid = nil
      if RUBY_PLATFORM.downcase =~ /mswin(?!ce)|mingw|cygwin|bccwin/
        yield
      else
        pid = fork do
          yield
        end
      end
      pid
    end

  end
end

