require 'pp'
require 'yaml'
require 'fileutils'
require 'open3'
require 'push_exchanger'

class Print001
  include PushExchanger::Util

  def load_plugin
    map = {}
    dirs = @pr_conf[:plugin_load_path] || []
    dirs.each do |dir|
      Dir.glob(File.join(dir,"*/")) do |plugin_dir|
        begin
          name = File.basename(plugin_dir)
          classname = name.dup
          classname[0] = classname[0].upcase
          next if map[name]
          require "#{plugin_dir}/#{name}.rb"
          map[name] = Module.const_get("#{classname}").new
          @logger.info("print001 plugin #{name} loaded")
        rescue Exception => ex
          @logger.info("print001 plugin #{name} load error")
          @logger.info ex.message
        end
      end
    end
    map
  end

  def initialize(conf)
    @conf = conf
    @logger = @conf[:logger]
    filename = File.join(File.dirname(File.expand_path(__FILE__)),"print001.yml")
    begin
      @pr_conf = YAML.load(File.read(filename))
      @plugin_map = load_plugin
      @pr_map = {}
      @pr_conf[:printer_settings].each do |k,v|
        n = 1
        if /(.*)#(\d+)$/ =~ k
          k = $1.to_sym
          n = $2.to_i
        end
        @pr_map[k] = {
          :printer => v,
          :num     => n
        }
      end
    rescue Exception
      @logger.info("print001 init failure: could not read config")
    end
  end

  def get_events
    if @pr_conf
      %w|print001|
    else
      []
    end
  end

  def print001(data)
    @logger.info("print001 start")

    body = data["body"]
    user = data["user"]
    enable_users  = @pr_conf[:enable_users]
    disable_users = @pr_conf[:disable_users]

    #check user
    if enable_users
      unless enable_users.include?(user)
        @logger.info("user[#{user}] is not in enable_users")
        return
      end
    end

    if @pr_conf[:disable_users]
      if disable_users.include?(user)
        @logger.info("user[#{user}] is in disable_users")
        return
      end
    end

    body.each do |r|
      begin
        print_report(r)
      rescue Exception => ex
        @logger.info("print report failure #{r}")
        @logger.info(ex)
      end
    end

    @logger.info("print001 end")
  end

  def print_report(r)
    did   = r["Data_ID"]  || ""
    rid   = r["Form_ID"] || r["Report_ID"] || ""
    cid   = r["Custom_ID"] || ""
    rname = r["Form_Name"] || r["Report_Name"] || ""
    return if did.empty? or rid.empty?
    json = get_report_data(rid,did)
    data = JSON.parse(json)
    result = data['Api_Result'].to_i
    unless result == 0
      @logger.info "api_error Api_Result[#{result}]"
      return
    end

    pr = @pr_map[rid.to_sym] || @pr_map[:default]
    unless pr
      @logger.info "no printer for #{rid}"
      return
    end
    printer = pr[:printer]
    num     = pr[:num]

    limit = @pr_conf[:report_limit] || 100
    dir =   @pr_conf[:report_dir] || "/var/tmp/print001"
    pdfname = Time.now.strftime("#{rid}-%Y%m%d%H%M%S-#{SecureRandom.uuid}.pdf")
    pdfname = File.join(dir,pdfname)
    opt = {:conf => @conf, :pr_conf => @pr_conf}

    file_logger(dir,pdfname,limit) do
      id = cid.empty? ? rid : cid
      pobj = @plugin_map[id]
      if pobj
        pobj.dup.export(pdfname,data,opt)
        print(pdfname,printer,num,@pr_conf[:print_command])
      else
        @logger.info("print001 #{id} plugin does not exist")
      end
    end
  end

  def get_report_data(rid,did)
    req = {
      "data" => {
        "Data_ID" => did
      }
    }
    res = http_post(@conf,"/api01rv2/formdatagetv2",req.to_json)
	unless res.code == "200"
      raise "api /api01rv2/formdatagetv2 error #{res.code}"
    end

    # save report data
    dir =   @pr_conf[:report_data_dir] || "/var/tmp/print001-data"
    limit = @pr_conf[:report_data_limit] || 100
    fname = Time.now.strftime("#{rid}-%Y%m%d%H%M%S-#{SecureRandom.uuid}.dat")
    fname = File.join(dir,fname)

    file_logger(dir,fname,limit) do
      File.open(fname,"wb") do |io| io.print res.body end
    end

    return res.body
  end

end
